/* ceelog event sources, providing events in order.

Copyright (C) 2012 Red Hat, Inc. All rights reserved.
This copyrighted material is made available to anyone wishing to use, modify,
copy, or redistribute it subject to the terms and conditions of the GNU General
Public License v.2.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
Street, Fifth Floor, Boston, MA 02110-1301, USA.

Author: Miloslav Trmač <mitr@redhat.com> */
#include <config.h>

#include <glib.h>

#include <ceelog/source.h>

#include "source_private.h"

/* Allocate and return a new source with OPS */
struct ceelog_source *
ceelog_source_alloc (const struct ceelog_source_ops *ops)
{
  struct ceelog_source *src;

  src = g_new (struct ceelog_source, 1);
  src->ops = ops;
  return src;
}

/* Free SOURCE. */
void
ceelog_source_free (struct ceelog_source *source)
{
  g_return_if_fail (source != NULL);

  source->ops->close (source);
  g_free (source);
}

/* Read the next event from SOURCE.  If no event is currently available, but
   it may arrive later, block until it is available.

   Return the event (for ceelog_event_free () if it is available, or NULL if
   no more events will arrive (without setting ERROR), or NULL on error. */
struct ceelog_event *
ceelog_source_read_event (struct ceelog_source *source, GError **error)
{
  g_return_val_if_fail (source != NULL, NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  return source->ops->read_event (source, error);
}

/* Read the next event from SOURCE.  If no event is currently available,
   fail with CEELOG_ERROR_WOULD_BLOCK.

   Return the event (for ceelog_event_free () if it is available, or NULL if
   no more events will arrive (without setting ERROR), or NULL on error. */
struct ceelog_event *
ceelog_source_read_event_nonblocking (struct ceelog_source *source,
				      GError **error)
{
  g_return_val_if_fail (source != NULL, NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  return source->ops->read_event_nonblocking (source, error);
}

/* Open the system default event source.
   Return the event source (for ceelog_source_free ()), or NULL on error.
   Usually, this is the function applications should be using. */
struct ceelog_source *
ceelog_source_new_default (GError **error)
{
  /* FIXME: Use a source that can handle live log files.  Support mongodb input,
     based on a configuration file. */
  return ceelog_source_new_static_file ("/var/log/messages", error);
}
