/* ceelog(1) command-line utility.

Copyright (C) 2012 Red Hat, Inc. All rights reserved.
This copyrighted material is made available to anyone wishing to use, modify,
copy, or redistribute it subject to the terms and conditions of the GNU General
Public License v.2.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
Street, Fifth Floor, Boston, MA 02110-1301, USA.

Author: Miloslav Trmač <mitr@redhat.com> */
#include <config.h>

#include <stdio.h>
#include <stdlib.h>

#include <glib.h>
#include <glib/gi18n.h>
#include <libintl.h>

#include <ceelog/source.h>
#include <ceelog/event.h>
#include <ceelog/expression.h>

/* Print an error message FMT, ... (without trailing '\n') to stderr, and
   exit (EXIT_FAILURE). */
G_GNUC_NORETURN
static void
error_exit (const char *fmt, ...)
{
  va_list ap;

  fprintf (stderr, _("%s: "), g_get_prgname ());
  va_start (ap, fmt);
  vfprintf (stderr, fmt, ap);
  va_end (ap);
  fputc ('\n', stderr);
  exit (EXIT_FAILURE);
}

int
main (int argc, char *argv[])
{
  GError *error;
  struct ceelog_compiled_expr *expr;
  struct ceelog_source *src;
  struct ceelog_event *evt;

  setlocale (LC_ALL, "");
  textdomain (PACKAGE_NAME);
  bindtextdomain (PACKAGE_NAME, LOCALEDIR);

  /* Eventually this should do much more. */
  g_set_prgname ("ceelog");
  if (argc != 2)
    error_exit (_("Usage: %s FILTER_EXPRESSION"), g_get_prgname ());

  expr = ceelog_parse_and_compile_expr (argv[1], &error);
  if (expr == NULL)
    error_exit (_("Error compiling %s: %s"), argv[1], error->message);

  src = ceelog_source_new_default (&error);
  if (src == NULL)
    error_exit (_("Error opening event source: %s"), error->message);

  while ((evt = ceelog_source_read_event (src, &error)) != NULL)
    {
      if (ceelog_eval_expr (expr, evt))
	{
	  char *text;

	  text = ceelog_event_to_string (evt);
	  printf ("%s\n", text);
	  g_free (text);
	}
      ceelog_event_free (evt);
    }
  if (error != NULL)
    error_exit (_("Error reading events: %s"), error->message);

  ceelog_source_free (src);

  ceelog_compiled_expr_free (expr);

  return EXIT_SUCCESS;
}
